{ A Basic mode $13 VGA graphics unit for the purposes of
  comparing a hand written VGA interface to the BGI Graph
  unit commonly included with Turbo Pascal. It's not
  designed to be as fast as possible, but it should be quick
  enough for the purposes of comparison.

  This version has been optimised in assembly partly, but
  I'm sure it could be done faster if you really want.

  I've used fixed point math so it should run ok on 286 and 386
  machines without a FPU.

  A Danson 2015 }

unit vga;

interface

{basic start-up and shutdown}
procedure init;
procedure shutdown;

{basic drawing stuff - lines and boxes}
procedure line(x,y,x1,y1 : integer; c : byte);
procedure filledBox(x,y,x1,y1: integer; c : byte);
procedure putPixel(x,y : integer; c : byte);

procedure circle(x,y,radius: word; colour: byte);

{basic blitting stuff - get and put in gwbasic terms}
{this will tell you how much memory to allocate to a
 pointer for a given screen area}
function imageSize(sx,sy:integer):integer;
procedure getImage(x,y,x1,y1 : integer; bitmap : pointer);
procedure putImage(x,y : integer; bitmap : pointer);


implementation

uses fixed;

const
     cardSeg = $A000;

var
   inited : boolean; {have we started the display?}
   exitSave : pointer;

procedure fillmem(sgm, ofs:word; d: byte; count:word); assembler;
asm
   {set up the counter}
   mov cx, count
   shr cx, 1
   {set the direction}
   cld
   {load data in the AX register}
   mov al, d
   mov ah, al
   {load the destination pointer}
   mov bx, sgm
   mov es, bx
   mov bx, ofs
   mov di, bx
   {use rep stosw to store the data}
   rep stosw
   {check if there is one last byte to copy (an odd numbered count)}
   {luckily the carry bit isn't touched from the shr earlier}
   jnc @done
   stosb
@done:
end;

procedure copymem(srcseg,srcofs,destseg,destofs,count:word); assembler;
asm
    push ds
    {load the count and divide by 2}
    mov cx, count
    shr cx, 1
    {set direction}
    cld
    {load the source pointer}
    mov bx, srcseg
    mov ds, bx
    mov bx, srcofs
    mov si, bx
    {load the destination pointer}
    mov bx, destseg
    mov es, bx
    mov bx, destofs
    mov di, bx
    {use movsw to copy the data}
    rep movsw
    {copy the last byte if needed (an odd count)}
    {luckily the carry flag is unchanhed from the shr}
    jnc @done
    movsb
@done:
    pop ds
end;

function checkBounds(x,y:integer):boolean; {returns true on failure!}
begin
    checkBounds := not(inited);
    if ((x<0) or (x>319)) then checkBounds := true;
    if ((y<0) or (y>199)) then checkBounds := true;
end;

{basic start-up and shutdown}
procedure init;
begin
     if (inited) then exit;
     asm
        mov ax, $0013
        int $10
     end;
     inited := true;
     exitSave := exitproc;
     exitproc := @shutdown;
     fillmem(cardseg,0,0,64000);
end;

{$f+}
procedure shutdown;
begin
     if not(inited) then exit;
     asm  {the small amount of assembler I will use}
        mov ax, $0003
        int $10
     end;
     inited := false;
     exitproc := exitsave;
end;
{$f-}

{basic drawing stuff - lines and boxes}

function sign(i : integer): integer;
begin
    sign :=0;
    if i>0 then sign:=1;
    if i<0 then sign:=-1;
end;

procedure line(x,y,x1,y1: integer; c: byte);
var
   cx,cy,count,xs,ys,xm,ym:integer;
begin
    cx := x; cy := y * 320;
    xs := x1 - x; ys := y1-y;

    xm:= sign(xs); ym := sign(ys) * 320;
    xs := abs(xs); ys := abs(ys);

    putpixel(cx,cy,c);

    if xs > ys then
    begin
        count := -(xs div 2);
        while (cx <> x1) do
        begin
            count := count + ys;
            cx:= cx + xm;
            if count>0 then
            begin
                 cy:=cy+ym;
                 count := count - xs;
            end;
            mem[cardseg: cx + cy] := c;
        end;
    end
    else
    begin
         count := -(ys div 2);
         y1 := y1 * 320;
         while (cy <> y1) do
         begin
             count := count + xs;
             cy := cy + ym;
             if count>0 then
             begin
                  cx:=cx + xm;
                  count := count - ys;
             end;
             mem[cardseg: cx + cy] := c;
         end;
    end;
end;

{ commented out in favour of another algorithm
procedure line(x,y,x1,y1: integer; c: byte);
var
   grad : integer; {gradient of the line
   rise : integer;
   run  : integer;
   i,z : integer;
   ofs : word;
begin
   if checkBounds(x,y) then exit;
   if checkBounds(x1,y1) then exit;
   if (x=x1) then {check for vertical line!
   begin
        z:= abs(y-y1);
        if y>y1 then y:=y1;
        for i:= 0 to z do
            mem[cardseg:((y+i)*320)+x] := c;
        exit;
   end;
   if (y=y1) then {check for a horizontal line!
   begin
       z:= abs(x-x1) + 1; { compute the number of pixels to draw
       if (x<x1) then ofs := x else ofs := x1;
       ofs:= (320*y) + ofs;
       fillmem(cardseg, ofs, c, z);
   end;
   if (x>x1) then
   begin {bring about some sanity, order the points according to x
         i:= x;
         x:= x1;
         x1 := i;
         i:= y;
         y:=y1;
         y1:=i;
   end;
   rise := (intToFixed(y)-intToFixed(y1));
   run :=  (intToFixed(x)-intToFixed(x1));
   if (abs(rise)>abs(run)) then
   begin
      if (y>y1) then
      begin {bring about some sanity, order the points according to y
            i:= x;
            x:= x1;
            x1 := i;
            i:= y;
            y:=y1;
            y1:=i;
            rise := (intToFixed(y)-intToFixed(y1));
            run :=  (intToFixed(x)-intToFixed(x1));
      end;
      grad := fixedDiv(run,rise);
      z:= intToFixed(x);
      for i:= y to y1 do
      begin
          z:= z + grad;
          mem[cardseg:(i*320)+fixedToInt(z)] := c;
      end;
   end
   else
   begin
       grad := fixedDiv(rise,run);
       z:= intToFixed(y);
       for i:= x to x1 do
       begin
            z:= z + grad;
            mem[cardseg:(fixedToInt(z)*320)+i] := c;
       end;
   end;

end;
end commented out section!}

procedure filledBox(x,y,x1,y1: integer; c : byte);
var
   tmp : integer;
   i,z : integer;
   ofs : word;
begin
   if checkBounds(x,y) then exit;
   if checkBounds(x1,y1) then exit;
   if (x>x1) then
   begin
        tmp := x;
        x:= x1;
        x1 := tmp;
   end;
   if (y>y1) then
   begin
        tmp := y;
        y:= y1;
        y1 := tmp;
   end;
{   for z:=y to y1 do
   begin
        ofs:= (z*320) + x;
        fillmem(cardSeg,ofs, c, (x1-x)+1);
   end;}
   asm
       {load the destination pointer}
       mov bx, cardseg
       mov es, bx

       mov ax, y
       mov bx, 320
       mul bx
       add ax, x
       mov di, ax

       {set up bx with the width of the box}
       mov bx, x1
       sub bx, x
       add bx, 1

       {set up cx to count the number of rows}
       mov cx, y1
       sub cx, y
       add cx, 1

       {load the AX register with the word data}
       mov al, c
       mov ah, al

       {set the direction}
       cld

       {loop and copy the data!}
       @row:
       push cx
       mov cx, bx
       shr cx, 1 {divide cx by two to copy words}

       {copy the accumulator to memory repeatedly}
       rep stosw

       jnc @even
       stosb
       @even:

       {end loop - work out next start point}
       add di, 320
       sub di, bx
       pop cx
       loop @row
   end;
end;

procedure putPixel(x,y : integer; c : byte);
begin
    if checkBounds(x,y) then exit;
    mem[cardSeg: x + (y*320)] := c;
end;

{basic blitting stuff - get and put in gwbasic terms}
{this will tell you how much memory to allocate to a
 pointer for a given screen area}
function imageSize(sx,sy:integer):integer;
begin
     imageSize := sx * sy + 4;
end;

procedure getImage(x,y,x1,y1 : integer; bitmap : pointer);
var
   bseg,bofs : word;
   i : word;
   sizex,sizey : word;
begin
     if checkBounds(x,y) then exit;
     if checkBounds(x1,y1) then exit;
     bseg := seg(bitmap^);
     bofs := ofs(bitmap^);
     sizex := abs(x-x1) + 1;
     sizey := abs(y-y1);
     memw[bseg:bofs] := sizex;
     memw[bseg:bofs+2] := sizey;
     if y>y1 then y:=y1;
     if x>x1 then x:=x1;
     {for i:= 0 to sizey do
     begin
          copymem(cardseg, ((y+i) * 320) + x, bseg, 4+(i*sizex), sizex);
     end;}
     {replacing the above with assembly!}
     asm
        push ds
        {set up the counters for the loop}
        mov cx, sizey
        inc cx

        {set up the segment pointers}
        mov ax, cardseg
        mov ds, ax
        mov ax, bseg
        mov es, ax
        {set up the destination offset}
        mov ax, bofs
        add ax, 4
        mov di, ax
        {work out the first row starting ofset}
        mov ax, y
        mov dx, 320
        mul dx
        add ax, x
        mov si, ax
        mov ax, sizex

        {start of the loop}
        @startrow:
        {do the transfer}
        push cx
        mov cx,ax
        shr cx, 1

        rep movsw

        jnc @even
        movsb
        @even:

        pop cx
        {update si for next row}
        add si, 320
        sub si, ax
        loop @startrow
        pop ds
     end;
end;

procedure putImage(x,y : integer; bitmap : pointer);
var
   bseg,bofs : word;
   i : word;
   sizex,sizey : word;
begin
     if checkBounds(x,y) then exit;
     bseg := seg(bitmap^);
     bofs := ofs(bitmap^);
     sizex := memw[bseg:bofs];
     sizey := memw[bseg:bofs+2];
     if (x+sizex>319) then
        sizex := 320-x;
     if (y+sizey>199) then
        sizey := 200-y;
     {for i:= 0 to sizey do
         copymem(bseg,4+(i*sizex),cardseg,((y+i)*320)+x, sizex); }
     {replaced pascal loop with assembly!}
     asm
        push ds
        {set up the counters for the loop}
        mov cx, sizey
        inc cx

        {set up the segment pointers}
        mov ax, cardseg
        mov es, ax
        mov ax, bseg
        mov ds, ax
        {set up the destination offset}
        mov ax, bofs
        add ax, 4
        mov si, ax
        {work out the first row starting ofset}
        mov ax, y
        mov dx, 320
        mul dx
        add ax, x
        mov di, ax
        mov ax, sizex

        {start of the loop}
        @startrow:
        {do the transfer}
        push cx
        mov cx,ax
        shr cx,1

        rep movsw

        jnc @even
        movsb
        @even:

        pop cx
        {update di for next row}
        add di, 320
        sub di, ax
        loop @startrow
        pop ds
     end;
end;

procedure swap(var a,b:word);
var
   x:word;
begin
     x:= a;
     a:= b;
     b:= x;
end;

procedure circle(x,y,radius: word; colour: byte);
{circle algorithm from SWAG that uses integers}
var
   a, af, b, bf,
   target, r2 : word;
   yofs : word; {the y offset into the vga memory}
begin
     target := 0;
     a := radius;
     b := 0;
     r2 := radius * radius;

     while (a>=b) do
     begin
          b:= intsqrt(R2 - (a*a)) ;
          swap(target, b);
          while (b<target) do
          begin
              {af := (120 * a) div 100;  Aspect correction results in
              bf := (120 * b) div 100;   missing stripes}

              {new aspect correction now changes the y value}
              {also uses 4:3 aspect ratio}
              {now also includes calculation for memory ofset}
              af:= ((a * 3) div 4) * 320 ;
              bf:= ((b * 3) div 4) * 320 ;
              yofs := y * 320;

              {optimise the drawing with asm}
              asm
                 {set up the es:di initial value}
                 mov ax, cardseg
                 mov es, ax
                 mov ax, x
                 add ax, yofs
                 mov di, ax
                 mov bx, ax  {store in bx for later use as well.}
                 mov cx, a   {store a in cx to reduce memory usage}
                 mov dx, b   {store b in dx for the same reason}
                 {put the colour in al}
                 mov al, colour
                 {draw +a +bf}
                 add di, cx
                 add di, bf
                 mov [es:di], al
                 mov di, bx
                 {draw +b +af}
                 add di, dx
                 add di, af
                 mov [es:di], al
                 mov di, bx
                 {draw -a +bf}
                 sub di, cx
                 add di, bf
                 mov [es:di], al
                 mov di, bx
                 {draw -b +af}
                 sub di, dx
                 add di, af
                 mov [es:di], al
                 mov di, bx
                 {draw -a -bf}
                 sub di, cx
                 sub di, bf
                 mov [es:di], al
                 mov di, bx
                 {draw -b -af}
                 sub di, dx
                 sub di, af
                 mov [es:di],al
                 mov di, bx
                 {draw +a - bf}
                 add di, cx
                 sub di, bf
                 mov [es:di], al
                 mov di, bx
                 {draw +b - af}
                 add di, dx
                 sub di, af
                 mov [es:di],al
              end;

              {
              mem[cardseg: x + a + yofs + bf]:= colour;
              mem[cardseg: x + b + yofs + af]:= colour;
              mem[cardseg: x - a + yofs + bf]:= colour;
              mem[cardseg: x - b + yofs + af]:= colour;
              mem[cardseg: x - a + yofs - bf]:= colour;
              mem[cardseg: x - b + yofs - af]:= colour;
              mem[cardseg: x + a + yofs - bf]:= colour;
              mem[cardseg: x + b + yofs - af]:= colour;
              }

              {putpixel(x+a,y+bf,colour);
              putpixel(x+b,y+af,colour);
              putpixel(x-a,y+bf,colour);
              putpixel(x-b,y+af,colour);
              putpixel(x-a,y-bf,colour);
              putpixel(x-b,y-af,colour);
              putpixel(x+a,y-bf,colour);
              putpixel(x+b,y-af,colour);}
              inc(b);
          end;
          dec(a);
     end;
end;

begin
     inited := false;
end.