{ VGA Benchmarking program }
{ A Danson 2015 }

{ This program is the companion to BGIBench in the sense that it
  performs similar measurements with a hand written VGA unit
  instead of the graph unit + BGI driver

  As such this will only work on VGA cards}

program vgabench;

uses vga, time;

var
   driver : string;
   driverString : string;
   mode   : integer;
   modeString : string;
   maxX   : integer;
   maxY   : integer;
   maxColour : integer;
   maxMode : integer;
   maxModeString : string;
   sprites : real;
   spriteSize : integer;
   vlines  : real;
   hlines  : real;
   rlines  : real;
   circles : real;
   fboxes  : real;
   pboxes  : real;
   pixels  : real;
   btext   : real;
   stext   : real;
   littfhandle : integer;

procedure initdisplay;
var
   i : integer;
begin

     driver := 'VGA.TPU';

     init;
     maxX := 320;
     maxY := 200;
     maxColour := 256;
     maxMode := 0;
     modeString := 'VGA 13H 320x200 256 colours';
     maxModeString := 'One mode only!';
     driverString := 'Hand coded VGA graphics stuff';
end;


procedure bench_sprites;
var
   sprite : pointer;
   size : integer;
   x,y,i :integer;
   count : longint;
   start, stop : single;
   rate : real;
begin
     randomize;
     for x:=0 to 9 do
         for y:= 0 to 9 do
             putPixel(x,y, random(maxColour));

     size := imageSize(10,10);
     spriteSize:=size;
     getMem(sprite, size);
     getImage(0,0,9,9,sprite);

     count:=0;
     start := timer;
     stop := start + 30;
     while (timer<stop) do
     begin
          for i:= 0 to 29 do
              putImage(random(maxX),random(maxY), sprite);
          count := count + 30;
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     sprites := rate;
     freeMem(sprite, size);
end;

procedure bench_lines;
var
   i,z,c : integer;
   count : longint;
   start, stop : single;
   rate : real;
begin
     {vertical line test}
     c:=0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 9 do
              for z:= 0 to 19 do
                   line(i+(z*10),0,i+(z*10),199,c);
          count := count +  200;
          if c=maxColour-1 then c:=0 else inc(c);
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     vlines := rate;

     {horizontal line test}
     c:=0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 9 do
              for z:= 0 to 19 do
                   line(0,i+(z*10),200,i+(z*10),c);
          count := count +  200;
          if c=maxColour-1 then c:=0 else inc(c);
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     hlines := rate;

     {random line test}
     c:=0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 49 do
              line(random(maxX),random(maxY),random(maxX),random(maxY),c);
          count := count +  50;
          if c=maxColour-1 then c:=0 else inc(c);
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     rlines := rate;
end;

procedure bench_circles;
var
   i,z,c : integer;
   cx,cy : integer;
   count : longint;
   start, stop : single;
   rate : real;
begin
     count := 0;
     c:=0;
     cx := maxX div 2;
     cy := maxY div 2;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 49 do
          begin
              circle(cx,cy, (i+20), (c+i) mod maxColour);
          end;
          count := count +  50;
          if c=maxColour-1 then c:=0 else inc(c);
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     circles := rate;
end;

procedure bench_boxes;
var
   i,z,c,o : integer;
   count : longint;
   start, stop : single;
   rate : real;
begin
     {filled boxes test}
     count := 0;
     start := timer;
     stop := start + 30;
     o:=0;
     while timer<stop do
     begin
          for i:= 0 to 19 do
              for c:= 0 to 19 do
              begin
                   z:= (c + o + (i*5)) mod maxColour;
                   filledbox(i*10,c*10,(i*10)+9,(c*10)+9,lo(z));
              end;
          inc(o);
          count := count +  400;
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     fboxes := rate;

     {pattern filled boxes test}
     {  not implemented
     count := 0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 19 do
              for c:= 0 to 19 do
              begin
                   z:= (c + o + (i*5)) mod maxColour;
                   setFillStyle(HatchFill, z);
                   bar(i*10,c*10,(i*10)+9,(c*10)+9);
              end;
          inc(o);
          count := count +  400;
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     pboxes := rate; }
end;

procedure bench_pixels;
var
   i : integer;
   count : longint;
   start, stop : single;
   rate : real;
begin
     {pixel test}
     count := 0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 399 do
              putpixel(random(maxX),random(maxY),random(maxColour));
          count := count +  400;
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     pixels := rate;
end;

{ not implemented
procedure bench_text;
var
   i : integer;
   count : longint;
   start, stop : single;
   rate : real;
begin
     littfhandle := registerBGIFont(@littfont);

     setTextStyle(DefaultFont, HorizDir, 1);

     bitmap font test
     count := 0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 99 do
          begin
               setColor(i mod maxColour);
               outtextxy(random(maxX),random(maxY),'Test Text');
          end;
          count := count +  100;
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     btext := rate;

     setTextStyle(littfhandle, HorizDir, 4);

     bitmap font test
     count := 0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 99 do
          begin
               setColor(i mod maxColour);
               outtextxy(random(maxX),random(maxY),'Test Text');
          end;
          count := count +  100;
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     stext := rate;

end; }

begin

    initDisplay;

    bench_sprites;
    bench_lines;
    bench_circles;
    bench_boxes;
    bench_pixels;
    {bench_text;}

    shutdown;
    writeln('results for ',driver);
    writeln(' driver string ', driverString);
    writeln(' mode:',mode,':',modeString);
    writeln(' maxMode:',maxMode,':',maxModeString);
    writeln(' resolution ',maxX,'x',maxY);
    writeln(' colours ',maxColour);
    writeln(' 10x10 sprite memory usage ',spriteSize);
    writeln;
    str(sprites:3:3,driver);
    writeln(' sprites/second ',driver);
    str(vlines:3:3,driver);
    writeln(' vertical lines/second ',driver);
    str(hlines:3:3,driver);
    writeln(' horizontal lines/second ',driver);
    str(rlines:3:3,driver);
    writeln(' random lines/second ',driver);
    str(circles:3:3,driver);
    writeln(' circles/second ',driver);
    str(fboxes:3:3,driver);
    writeln(' filled boxes/second ',driver);
    {str(pboxes:3:3,driver);
    writeln(' pattern filled boxes/second ',driver); }
    str(pixels:3:3,driver);
    writeln(' put-pixels/second ',driver);
    { str(btext:3:3,driver);
    writeln(' bitmapped text/second ',driver);
    str(stext:3:3,driver);
    writeln(' stroked text/second ',driver); }
end.